--[[
Name: MythicPlayerInfo

Copyright 2018 Amegez,MerguezChipo

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
]]

ACTIVITY_IDS={};
ACTIVITY_IDS[197]="EoA";
ACTIVITY_IDS[198]="DT";
ACTIVITY_IDS[199]="BRH";
ACTIVITY_IDS[200]="HoV";
ACTIVITY_IDS[206]="NL";
ACTIVITY_IDS[207]="VoW";
ACTIVITY_IDS[208]="MoS";
ACTIVITY_IDS[209]="Arc";
ACTIVITY_IDS[210]="CoS";
ACTIVITY_IDS[227]="RKL";
ACTIVITY_IDS[233]="CoEN";
ACTIVITY_IDS[234]="RKH";
ACTIVITY_IDS[239]="SoT";
ACTIVITY_IDS[244]="AD";
ACTIVITY_IDS[245]="FH";
ACTIVITY_IDS[246]="TD";
ACTIVITY_IDS[247]="ML";
ACTIVITY_IDS[248]="WM";
ACTIVITY_IDS[249]="KR";
ACTIVITY_IDS[250]="TOS";
ACTIVITY_IDS[251]="UNDR";
ACTIVITY_IDS[252]="SOTS";
ACTIVITY_IDS[353]="SIEGE";

local MythicPlayerInfo = {};

C_ChatInfo.RegisterAddonMessagePrefix("MythicPlayerInfo");
local defaultGTTFontName = "";
local defaultGTTFontSize = 0;

local f = CreateFrame("Frame")
f:SetScript("OnEvent", function(e,EventName,Data1,Data2)
	if(EventName == "CHAT_MSG_ADDON_LOGGED" and Data1 == "MythicPlayerInfo")then
		MPI_setPlayerInfo(Data2);
	elseif (EventName == "LFG_LIST_APPLICATION_STATUS_UPDATED" and Data2 == "applied")then
		local _, _, _, _, _, _, _, _, _, _, _, _, leader = C_LFGList.GetSearchResultInfo(Data1);
		C_ChatInfo.SendAddonMessageLogged("MythicPlayerInfo", MPI_GetPlayerInfo(), "WHISPER", leader);
	elseif (EventName == "PLAYER_ENTERING_WORLD")then
		C_MythicPlus.RequestCurrentAffixes();
		C_MythicPlus.RequestMapInfo();
		C_MythicPlus.RequestRewards();
		MPI_SaveGameTooltipFont();
	end
end)
f:RegisterEvent("CHAT_MSG_ADDON_LOGGED");
f:RegisterEvent("LFG_LIST_APPLICATION_STATUS_UPDATED");
f:RegisterEvent("PLAYER_ENTERING_WORLD");

function MPI_ShowToolTip(self)
	local name, class, localizedClass, level, itemLevel = C_LFGList.GetApplicantMemberInfo(self:GetParent().applicantID,1);
	--namer = string.format("%s-%s",name,GetRealmName());
	local classTextColor = grayedOut and GRAY_FONT_COLOR or RAID_CLASS_COLORS[class];
	local hasOwner = GameTooltip:GetOwner()
	if not hasOwner then
		GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT", 0, 0)
	end
	GameTooltip:ClearLines();
	GameTooltip:AddLine(name,classTextColor.r, classTextColor.g, classTextColor.b);
	GameTooltip:AddLine(string.format(UNIT_TYPE_LEVEL_TEMPLATE, level, localizedClass), 1, 1, 1);
	GameTooltip:AddLine(string.format(LFG_LIST_ITEM_LEVEL_CURRENT, itemLevel), 1, 1, 1);
	
	MPI_SetTooltipPlayerInfo(name,GameTooltip);

end

function MPI_HideToolTip(self)
	GameTooltip:Hide();
	MPI_RestoreGameTooltipFont();
	GameTooltip:ClearLines();
end

function MPI_setPlayerInfo(data)
	local info = {strsplit("#",data)};
	MythicPlayerInfo[info[1]] = {};
	for i=2,#info,4 do
		table.insert(MythicPlayerInfo[info[1]],{info[i],info[i+1],info[i+2],info[i+3]});
	end
end

function MPI_GetPlayerInfo()
	--local playerInfo = string.format("%s-%s",UnitFullName("player"));
	local playerName = UnitFullName("player");
	local playerInfo = playerName;
	for k,v in pairs(ACTIVITY_IDS) do
		local seasonBestDurationSec, seasonBestLevel, _, _, members = C_MythicPlus.GetSeasonBestForMap(k);
		if seasonBestLevel then
			local role = 0;
			for i, member in ipairs(members) do
				if (member.name and member.name == playerName) then
					role = member.specID;
				end
			end
			playerInfo = string.format("%s#%s#%s#%s#%s",playerInfo,k,seasonBestDurationSec,seasonBestLevel,role);
		end
	end
	return playerInfo;
end

function MPI_SetTooltipPlayerInfo(name,tooltip)
	local score = 0;
	local str = {};
	tooltip:AddLine("MythicPlayerInfo");
	if MythicPlayerInfo[name] then
		for k,v in pairs(MythicPlayerInfo[name]) do
			local mname,_,timer = C_ChallengeMode.GetMapUIInfo(v[1]);
			mname = MPI_formatString(mname,30);
			score = score + ((v[3]*10)+((timer-v[2])/(timer/10)));
			local oyu = 0;
			if timer-v[2] >= 0 then
				oyu = string.format("|cffabd473Under by %s|r",GetTimeStringFromSeconds(timer-v[2]));
			else
				oyu = string.format("|cffc41f3bOver by %s|r",GetTimeStringFromSeconds(v[2]-timer));
			end
			local dtime = GetTimeStringFromSeconds(v[2]);
			local _, name = GetSpecializationInfoByID(tonumber(v[4]));
			name = MPI_formatString(name,15);
			local role = GetSpecializationRoleByID(tonumber(v[4]));
			local texture = CreateAtlasMarkup("roleicon-tiny-dps");
			if (role == "TANK") then
				texture = CreateAtlasMarkup("roleicon-tiny-tank");
			elseif (role == "DAMAGER") then
				texture = CreateAtlasMarkup("roleicon-tiny-dps");
			elseif (role == "HEALER") then
				texture = CreateAtlasMarkup("roleicon-tiny-healer");
			end
			local lvel = MPI_formatString(v[3],2);
			str[k] = string.format("%s %s +%s %s (%s)",mname,MYTHIC_PLUS_LEADER_BOARD_NAME_ICON:format(texture,name),lvel,dtime,oyu);
		end
		tooltip:AddLine(string.format("Score : %.2f",score),0,0.8,0);
		for k,v in pairs(str) do
			tooltip:AddLine(v,1,1,1);		
		end
	else
		tooltip:AddLine("Score : "..score,1,1,1);
	end
	for i=1,select("#",GameTooltip:GetRegions())do
		local region=select(i,GameTooltip:GetRegions())
		if region and region:GetObjectType()=="FontString" and region:GetText()then
			region:SetFont("Interface\\AddOns\\MythicPlayerInfo\\fonts\\VeraMono.ttf", 12)
		end
	end	
	tooltip:Show();
end

function MPI_formatString(str, maxChar)
	local specialChars = strtrim(str,'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890 -_\'');
	local toRemove = floor(#str-(#specialChars/2));
	return str..string.rep(' ', maxChar-toRemove);
end

function MPI_ButtonShow(self)
	for i=1,#self.Members do
		local b = self.Members[i];
		b:HookScript("OnEnter", MPI_ShowToolTip);
		b:HookScript("OnLeave", MPI_HideToolTip);
	end
end

function MPI_SaveGameTooltipFont()
	for i=1,select("#",GameTooltip:GetRegions())do
		local region=select(i,GameTooltip:GetRegions())
		if region and region:GetObjectType()=="FontString" and region:GetText()then
			defaultGTTFontName, defaultGTTFontSize = region:GetFont();
		end
	end	
end

function MPI_RestoreGameTooltipFont()
	for i=1,select("#",GameTooltip:GetRegions())do
		local region=select(i,GameTooltip:GetRegions())
		if region and region:GetObjectType()=="FontString" and region:GetText()then
			region:SetFont(defaultGTTFontName, defaultGTTFontSize);
		end
	end	
end

for i=1,14 do
	local b = _G["LFGListApplicationViewerScrollFrameButton"..i];
	b:HookScript("OnShow", MPI_ButtonShow);
end
